﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using BookstoreSpy.Providers;
using System.Collections.Generic;
using System.Threading;

namespace BookstoreSpy.Classes
{
    public class MultiThreadedShopsExplorer : IShopsExplorer
    {
        private string isbn;
        private List<PriceResult> results = new List<PriceResult>();

        public string ISBN
        {
            get { return isbn; }
        }

        public List<PriceResult> Results
        {
            get { return results; }
        }

        private IBookstoreProvider[] getAvailableProviders()
        {
            return new IBookstoreProvider[] {
                new AmazingBookstoreProvider(),
                new BooksAllRightProvider(),
                new FredAndBarneyProvider(),
                new HundredBooksProvider() };
        }

        private bool isCompleted = false;
        public bool IsCompleted
        {
            get { return isCompleted; }
        }

        private DateTime startTime;
        public DateTime StartTime
        {
            get { return startTime; }
        }

        private DateTime endTime;
        public DateTime EndTime
        {
            get { return endTime; }
        }


        public MultiThreadedShopsExplorer(string isbn)
        {
            this.isbn = isbn;
        }

        public void GetPrices()
        {
            isCompleted = false;
            startTime = DateTime.Now;
            try
            {
                List<Thread> threads = new List<Thread>();
                foreach (IBookstoreProvider provider in getAvailableProviders())
                {
                    Thread thread = new Thread(new ParameterizedThreadStart(threadSafeDoProcess));
                    threads.Add(thread);
                    thread.Start(provider);
                }

                foreach (Thread thread in threads)
                {
                    thread.Join();
                }
            }
            finally
            {
                isCompleted = true;
                endTime = DateTime.Now;
            }
        }

        private void doProcess(object parameter)
        {
            IBookstoreProvider provider = parameter as IBookstoreProvider;
            if (provider != null)
            { 
                // ATTENZIONE: non è thread safe
                results.Add(provider.GetPrice(isbn));
            }
        }

        private object sync = new object();
        private void threadSafeDoProcess(object parameter)
        {
            IBookstoreProvider provider = parameter as IBookstoreProvider;
            if (provider != null)
            {
                PriceResult priceResult = provider.GetPrice(isbn);

                // ora bisogna memorizzare il dato

                // imposto il lock sull'oggetto sync
                lock (sync)
                {
                    // aggiungo il dato alla collection, solo un thread per volta
                    // potrà effettuare questa operazione
                    results.Add(priceResult);
                }
                
                // qui il lock è stato rilasciato
            }
        }
    }
}
