﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using BookstoreSpy.Providers;

namespace BookstoreSpy.Classes
{
    public class ShopsExplorer : IShopsExplorer
    {
        private string isbn;
        private List<PriceResult> results = new List<PriceResult>();

        public string ISBN
        {
            get { return isbn; }
        }

        public List<PriceResult> Results
        {
            get { return results; }
        }

        private IBookstoreProvider[] getAvailableProviders()
        {
            return new IBookstoreProvider[] {
                new AmazingBookstoreProvider(),
                new BooksAllRightProvider(),
                new FredAndBarneyProvider(),
                new HundredBooksProvider() };
        }

        private DateTime startTime;
        public DateTime StartTime
        {
            get { return startTime; }
        }

        private DateTime endTime;
        public DateTime EndTime
        {
            get { return endTime; }
        }

        private bool isCompleted = false;
        public bool IsCompleted
        {
            get { return isCompleted; }
        }

        public ShopsExplorer(string isbn)
        {
            this.isbn = isbn;
        }

        public void GetPrices()
        {
            startTime = DateTime.Now;
            try
            {
                foreach (IBookstoreProvider provider in getAvailableProviders())
                {
                    results.Add(provider.GetPrice(this.isbn));
                }
            }
            finally
            {
                endTime = DateTime.Now;
                isCompleted = true;
            }
            
        }
    }
}
